;Problem Statement  :  To design parser for English as natural language.
;Assumptions :
;(1)     The parser expects a single line input.
;(2)     The input statement is terminated by a space followed by period.
;(3)     It parses limited vocabulary.
;(4)     Ordinary nouns can be singular or plural.
;(5)     If ordinary noun is plural, the noun is followed by a space and 'S'.
;Description :
;This Top Down Parser parses the input line word by word using the specified grammar. For each phrase, there is a separate list in the grammar, for e.g. Noun list, Verb list. If it finds an error in the sentence, the program displays the error and terminates. It displays different error messages according to the part of the sentence in which error has occurred.
;Grammar For Natural Language Parser
;        <Starting_Symbol>       ::=     <Noun_Phrase> <Verb_Phrase>
;        <Noun_Phrase>   ::=     Determiner <Adjectival_Noun_Phrase>
;                                                        |<Adjectival_Noun_Phrase>
;        <Verb_Phrase>   ::=     <Transitive_Verb> <Noun_Phrase>
;                                                        <Adverb_List> <Prepositional_Phrase_List>
;                                                        |<Intransitive_Verb> <Adverb_List>
;                                                        <Prepositional_Phrase_List>
;
;
;        <Determiner>                            ::=     Articles | Possessive_Noun
;        <Adjectival_Noun_Phrase>        ::=     Adjective <Noun_Phrase>|
;                                                                        <Noun_Term>
;        <Adverb_List>                           ::=     Adverb <Adverb_List>| (
;        <Prepositional_Phrase_List>     ::=     <Prepositional_Phrase>
;                                                                        <Prepositional_Phrase_List>| (
;        <Noun_Term>                             ::=     <Number_Ordinary_Noun>|
;                                                                        <Proper_Noun>
;        <Prepositional_Phrase>          ::=     Preposition <Noun_Phrase>
;        <Number_Ordinary_Noun>  ::=     <Singular_Ordinary_Noun>|
;                                                                <Plural_Ordinary_Noun>
;        <Proper_Noun>                   ::=     Name | Pronoun
;        <Plural_Ordinary_Noun>          ::=     <Singular_Ordinary_Noun> S
;ALGORITHM :
;Step 1  :       Accept a line in form of string.
;Step 2  :       Prepare a list of words from the input string.
;Step 3  :       Call function CheckGrammer which parses the word list.
;CheckGrammer
;Step 1  :       Call function CheckNounPhrase
;Step 2  :       If no error, Call function Check VerbPhrase.
;CheckNounPhrase
;Step 1  :       Call function CheckDeterminer
;Step 2  :       If no error,
;Step 3  :       Advance to next word.
;Step 4  :       Call function CheckAdjectivalNounPhrase.
;CheckDeterminer
;Step 1  :       If word is Article or Possessive noun,
;Step 2  :       Return 1
;CheckVerbPhrase
;Step 1  :       Call function CheckAdverb
;Step 2  :       If found,
;Step 3  :       Advance to next word.
;Step 4  :       If no more words in the list, display error.
;Step 5  :       Call function CheckAdverbList
;Step 6  :       Else
;Step 7  :       Call function CheckPrepositionList.
;CheckAdverb
;Step 1  :       Search in the Adverb List
;Step 2  :       If found
;Step 3  :       return 1
;Step 4  :       Else
;Step 5  :       return 0
;CheckPrepositionList
;Step 1  :       Call function CheckPreposition
;Step 2  :       If found,
;Step 3  :       If more words in the list
;Step 4  :       Call function check preposition.
;CheckPreposition
;Step 1  :       Search in the Preposition List
;Step 2  :       If found
;Step 3  :       Advance to next word.
;Step 4  :       If no more words in the list, display error.
;Step 5  :       Call function CheckNounPhrase.
;CheckNumberOrdinaryNoun
;Step 1  :       Search in the SingleOrdinaryNoun List
;Step 2  :       If found
;Step 3  :       return 1
;Step 4  :       Else
;Step 5  :       return 0
;CheckVerbPhrase
;Step 1  :       Call function CheckTransitiveVerb
;Step 2  :       If found,
;Step 3  :       Advance to next word.
;Step 4  :       If no more words, display error.
;Step 5  :       Call function CheckNounphrase.
;Step 6  :       Else
;Step 7  :       Search in the Intransitive Verb list.
;Step 8  :       If found, advance to next word.
;Step 9  :       If both not present, display error.
;Step 10         :       Call function CheckAdverbList
;CheckTransitiveVerb
;Step 1  :       Search in the Transitive Verb List
;Step 2  :       If found
;Step 3  :       return 1
;Step 4  :       Else
;Step 5  :       return 0
;CheckAdverbList
;Step 1  :       If found
;Step 2  :       return 1
;Step 3  :       Else
;Step 4  :       Return 0
;CheckAdjectivalNounPhrase
;Step 1  :       Call function CheckAdjective
;Step 2  :       If found, advance to next word.
;Step 3  :       If word list empty, display error message.
;Step 4  :       Else Call function CheckAdjectivalNounPhrase
;Step 5  :       Else
;Step 6  :       Call function CheckNounTerm.
;CheckNounTerm
;Step 1  :       Call function CheckNumberOrdinaryNoun
;Step 2  :       Call function CheckProperNoun.
;Step 3  :       Advance to next word.
;Step 4  :       If NumberOrdinaryNoun or ProperNoun found
;Step 5  :       If word is S (for plural)
;Step 6  :       Advance to next word.
;CheckProperNoun
;Step 1  :       Search in ProperNoun list or Pronoun list.
;Step 2  :       If found,
;Step 3  :       return 1
;Step 4  :       Else
;Step 5  :       return 0
;Step 6  :       Else return 0.
;
;-------------------------------------------

; Natrual Language Parser Using Lisp
;-------------------------------------------
;-------------------------------------------
; Syntactic Categories
(setf Sentence "") ;* Sentence Buffer
(setf TransVerb 	(list "READ" "WRITE" "CRASH" "ABORT" "RETRY" "IGNORE" "CONTINUE" "BOOT"))
(setf IntransVerb (list "IS" "SLEEP" "HANG" "TALK" "OVERREACT" "GO"))
(setf Article (list "A" "AN" "THE"))
(setf Possessive (list "MY" "YOUR" "HIS" "HER" "ITS" "OUR" "THEIR"))
(setf Adjective (list "BLUE" "DEEP" "BIG" "BUGFUL" "COLORLESS" "FUNNY"))
(setf Adverb (list 'SLOWLY" "QUICKLY" "WELL" "FURIOUSLY" "STRANGELY"))
(setf Preposition (list "AT" "ON" "TO" "IN" "FOR" "AROUND" "BEHIND" "TOWARDS" "BEYOND"))
(setf SingOrdNoun (list "BOOK" "MICROPROCESSOR" "IDEA" "CHIP" "BUS" "ROBOT" "COMPUTER"))
(setf ProperNoun (list "BILL" "ANDY" "STEVE" "JACK" "JILL" "MACBETH" "INDIA"))
(setf ProNoun (list "I" "YOU" "HE" "SHE" "IT" "WE" "THEY"))
; -------------------------------------------
; -------------------------------------------
; FUNCTION TO PUT A SPECIFIED NO. OF TABS
; -------------------------------------------
(defun put_tab (num)
(setf tab "  ")
(do ((a 1 (+ 1 a)))
((> a num))
(princ tab)
) 
)
;---------------------------------------------
; FUNCTION TO DISPLAY ERRORS AND NO OF ERRORS
;--------------------------------------------
(defun display (msg)
(princ "\n")
(put_tab 3)
(princ msg)
(read)
(dos "cls")
(exit)
)
; -------------------------------------------
; MAIN
; -------------------------------------------
(defun main ()

(dos "cls")
(princ "\n")
(put_tab 3 )
(princ "Natural Language parser")
(princ "\n")
(print) 			;* Print all lists.
(put_tab 1)
(princ "Enter the sentence terminated by a space followed by a `.'")
(princ "\n\n")
(put_tab 3)
(princ "Enter the line -> \n\n")
(read-line)
(setf oneline (read-line))
(accept_list oneline)
(checkgram)
(display "No Errors Found.")
(exit)
)
; -------------------------------------------
; FUNCTION TO CHECK THE GRAMMAR OF THE SENTENCE
; -------------------------------------------
(defun checkgram ()
(setf no_err (Checknp))
(cond ((eql no_err 0)
(display "Error in Noun Phrase!"))
(t
(setf no_err2 (Checkvp))
(cond ((> no_err2 0))
(t
(display "Error in Verb phrase!"))
)
)
 	)
)
; -------------------------------------------
; FUNCTION TO CHECK FOR A NOUN PHRASE
; -------------------------------------------
(defun Checknp ()
(setf no_err (Checkdet (car Sentence)))
(cond ((> no_err 0)
(setf Sentence (cdr Sentence))
(cond ((null Sentence)
(display "Incomplete Sentence - No NP after Article!"))
)
 )
)
(setf ret (Checkanp))
)
; -------------------------------------------
;FUNCTION TO CHECK FOR A DETERMINER
; -------------------------------------------
(defun Checkdet (det)
(setf found (member det Article : test equal))
(cond ((equal det (car found))
(setf p 1)
)
(t
(setf p 0)
) 
)
(setf found (member det Possessive :test equal ))
(cond ((equal det (car found))
(setf q 1))
				)
(t
(setf q 0))
)
(setf ret (+ p q))
)
; -------------------------------------------
; FUNCTION TO ACCEPT A SENTENCE FROM A USER
; -------------------------------------------
 (defun accept_list (oneline)
(setf ch (char oneline 0))
(setf ch_s (string ch))
(setf word "")
(do ((a 1 (+ 1 a ))) ((eql ch_s "."))

(cond ((equal ch_s "give space")
(setf word_1 (list word))
(cond ((equal Sentence "")
(setf Sentence (list word))
(setf word "")) 
( t
(setf Sentence
(append Sentence word_1)) 
(setf word "")
)
)
( t
(setf word (strcat word ch_s))
)
)
(setf ch (char oneline a ))
(setf ch_s (string ch))
)
)
; -------------------------------------------
;FUNCTION TO CHECK ADJECTIVAL NOUN PHRASE
; -------------------------------------------
 (defun Checkanp ()
(setf yes_adj (Checkadj (car Sentence)))
(cond ((> yes_adj 0)
(setf Sentence (cdr Sentence))
(cond ((null Sentence)
(display "Incomplete Sentence - 
No NP after ADJECTIVE!")
)
)
(setf ret (Checkanp))
)
(t (setf ret (Checknt)))
)
)
; -------------------------------------------
; FUNCTION TO CHECK FOR AN ADJECTIVE
; -------------------------------------------
(defun Checkadj (adject)
(setf found (member adject Adjective :test equal))
(cond ((equal adject (car found))
(setf ret 1))
(t
(setf ret 0)
)
)
)
; -------------------------------------------
; FUNCTION TO CHECK FOR NOUN
; -------------------------------------------
(defun checknt ()
(setf p (Checknon (car Sentence)))
(setf q (Checkpn (car Sentence )))
(setf Sentence (cdr Sentence))
(cond ((> p 0)
(if (equal (car Sentence) "S")
(setf Sentence (cdr Sentence ))
)
)	
)
(setf ret (+ p q))
)
; -------------------------------------------
; FUNCTION TO CHECK FOR PROPER NOUN
; -------------------------------------------
(defun Checkpn (prop_noun)
(setf found (member prop_noun ProperNoun :test equal))
(cond ((equal prop_noun (car found))
(setf r 1)
)
(t
(setf r 0)
)
)
(setf found (member prop_noun ProNoun :test equal))
(cond ((equal prop_noun (car found))

(setf s 1))
(  t
(setf s 0)
)
)
(setf q (+ r s))
)
; -------------------------------------------
; FUNCTION TO CHECK FOR ORDINARY NOUN
; -------------------------------------------
(defun Checknon (ord_noun)
(setf found (member ord_noun SingOrdNoun : test equal))
(cond ((equal ord_noun (car found))
(setf p 1))
( t
(setf p 0)
)
)
( setf ret p)
)
; -------------------------------------------
; FUNCTION TO CHECK THE VERB PHRASE
; -------------------------------------------
(defun Checkvp ()
(setf trans_verb (CheckTransVerb (car Sentence)))
(cond ((> trans_verb 0)
(setf Sentence (cdr Sentence))
(if (null Sentence)
(display "Incomplete Sentence - 
No NP after TRANSITIVE-VERB!")
)
(setf ret (Checknp))
(if (eql ret 0)
(display "Error in Noun phrase following TRANSITIVE VERB !")
)
)
( t
(setf found (member (car Sentence)
IntransVerb :test equal))
(cond ((equal (car Sentence) (car found))
(setf ret (+ ret 1))
(setf Sentence (cdr Sentence)))
(  t
(setf ret (+ ret 0))
)
    )
  )
)
(cond ((> ret 0))
(  t
(display "Error in VERB PHRASE - Undefined INTRANSITIVE-VERB !")
)
)
(setf av_1 (Checkav1))
)
; -------------------------------------------
;FUNCTION TO CHECK FOR A TRANSITIVE VERB
; -------------------------------------------
 defun CheckTransVerb (trans_v)
(setf found (member trans_v TransVerb :test equal))
(cond ((equal trnas_v (car found))
(setf ret 1))
(  t
(setf ret 0)
)
)
)
; -------------------------------------------
; FUNCTION TO PRINT ALL THE LISTS
; -------------------------------------------
(defun print ()
(princ "\n TRANSITIVE VERBS -> ")
(princ TransVerb)
(princ "\n INTRASITIVE VERBS -> ")
(princ IntransVerb)
(princ "\n ARTICLES -> ")
(princ Article)
(princ "\n POSSESSIVE NOUNS -> ")
(princ Possessive)
(princ "\n ADJECTIVES -> ")
(princ Adjective)
(princ "\n ADVERBS -> ")
(princ Adverb)
(princ "\n PREPOSITIONS -> ")
(princ Preposition)
(princ "\n SINGULAR NOUNS -> ")
(princ SingOrdNoun)
(princ "\n PROPER NOUNS -> ")
(princ ProperNoun)
(princ "\n PRONOUNS -> ")
(princ ProNoun)
(princ "\n\n")
)
; -------------------------------------------
; FUNCTION TO CHECK THE ADVERB LIST
; -------------------------------------------
(defun Checkav1 () 
(setf yes_adv (Checkadv (car Sentence)))
(cond ((> yes_adv 0)
(setf Sentence (cdr Sentence))
(cond ((null Sentence)
(setf ret 1))
(  t
(setf ret(Checkav1))
)
)
)
( t
(setf ret (Checkppl))
)
)
)
; -------------------------------------------
; FUNCTION TO CHECK FOR AN ADVERB
; -------------------------------------------
(defun Checkadv (adv)
(cond ((null adv)
(setf ret 1))
(  t
(setf found (member adv Adverb :test
equal))
(cond ((equal adv (car found))
(setf ret 1)
)
(  t
(setf ret 0)
)
)
)
)
)
; -------------------------------------------
; FUNCTION TO CHECK THE PREPOSITION LIST
; -------------------------------------------
(defun Checkppl ()
(setf ret (Checkpp (car Sentence)))
(cond ((> ret 0)
(cond ((null Sentence)
(setf ret1))
 ( t
(setf ret (Checkppl))
)
)
)
(  t
(display "ERROR IN PREPOSITIONAL PHRASE!")
)
)
)
; -------------------------------------------
; FUNCTION TO CHECK FOR A PREPOSITION
; -------------------------------------------
(defun Checkpp (prepo)
(cond ((null prepo)
(setf ret 1))

(  t
(setf found (member prepo Preposition :test equal ))
(cond ((equal prepo (car found))
(setf Sentence (cdr Sentence))
(  if (null Sentence)
 (display "Incomplete PREPOSITION !")
)
(setf ret (Checknp))
)
(  t
(setf ret 0)
)
)
)
)
)
; -------------------------------------------
; MAIN FUNCTION 
; -------------------------------------------
(main)
(exit)
; -------------------------------------------
